/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */

using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// Unit test fixture for FixedGridLayoutManager.
    ///
    /// Testing of FixedGridLayoutManager includes the accuracy testing of the layout algorithm,
    /// to verify that each parameter works independently.
    /// </summary>
    [TestFixture]
    public class FixedGridLayoutManagerUnitTests
    {

        /// <summary>
        /// The filename to persist to.
        /// </summary>
        private string filename = @"../../test_files/unittests/unittest-";

        /// <summary>
        /// FixedGridLayoutManager instance to test.
        /// </summary>
        private FixedGridLayoutManager manager = null;

        /// <summary>
        /// ShapeFactory instance to use.
        /// </summary>
        private ShapeFactory factory = null;

        /// <summary>
        /// RendererGraphics to use.
        /// </summary>
        private IGraphics graphics = null;

        /// <summary>
        /// Create new layout manager, shape factory and renderer graphics for each test.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            manager = new FixedGridLayoutManager();
            factory = new ShapeFactory();            
        }

        /// <summary>
        /// Remove the file and dispose the renderer graphics.
        /// </summary>
        [TearDown]
        protected void TearDown()
        {
            if (graphics != null)
            {
                graphics.Dispose();
            }
        }

        /// <summary>
        /// Create instance.
        /// </summary>
        [Test]
        public void Create()
        {
            manager = new FixedGridLayoutManager();
        }

        /// <summary>
        /// Create instance with colPadding and rowPadding.
        /// </summary>
        [Test]
        public void Create_ColPaddingRowPadding()
        {
            manager = new FixedGridLayoutManager(10, 10);
        }

        /// <summary>
        /// Create instance with negative colPadding.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeColPadding()
        {
            new FixedGridLayoutManager(-1, 10);
        }

        /// <summary>
        /// Create instance with negative rowPadding.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeRowPadding()
        {
            new FixedGridLayoutManager(10, -1);
        }

        /// <summary>
        /// Create instance with colPadding, rowPadding and textMargin.
        /// </summary>
        [Test]
        public void Create_ColPaddingRowPaddingTextMargin()
        {
            manager = new FixedGridLayoutManager(10, 10, 10);
        }

        /// <summary>
        /// Create instance with negative colPadding.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeColPadding2()
        {
            new FixedGridLayoutManager(-1, 10, 10);
        }

        /// <summary>
        /// Create instance with negative rowPadding.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeRowPadding2()
        {
            new FixedGridLayoutManager(10, -1, 10);
        }

        /// <summary>
        /// Create instance with negative textMargin.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeTextMargin()
        {
            new FixedGridLayoutManager(10, 10, -1);
        }

        /// <summary>
        /// Layout single dependency.
        /// </summary>
        [Test]
        public void Layout_SingleDependency()
        {
            Dependency dependency = new Dependency("Dependency Map Renderer");
            
            graphics = new SVGGraphicsAdapter(filename + "single_dependency.svg");
            
            manager.Layout(dependency, factory, graphics);
            graphics.Complete();
        }

        /// <summary>
        /// Layout dependency tree.
        /// </summary>
        [Test]
        public void Layout_DependencyTree()
        {
            Dependency dependency = new Dependency("Dependency Map Renderer");
            dependency.AddDependant(new Dependency("SVG Generator"));
            dependency.AddDependant(new Dependency("Base Exception"));

            graphics = new SVGGraphicsAdapter(filename + "dependency_tree.svg");

            manager.Layout(dependency, factory, graphics);
            graphics.Complete();
        }

        /// <summary>
        /// Layout seen dependency.
        /// </summary>
        [Test]
        public void Layout_SeenDependency()
        {
            Dependency dependency = new Dependency("Dependency Map Renderer");
            dependency.AddDependant(dependency);

            graphics = new SVGGraphicsAdapter(filename + "seen_dependency.svg");

            manager.Layout(dependency, factory, graphics);
            graphics.Complete();
        }

        /// <summary>
        /// Layout shapeOverride.
        /// </summary>
        [Test]
        public void Layout_ShapeOverride()
        {
            Dependency dependency = new Dependency("Dependency Map Renderer", ShapeFactory.SeenElement);

            graphics = new SVGGraphicsAdapter(filename + "shape_override.svg");

            manager.Layout(dependency, factory, graphics);
            graphics.Complete();            
        }

        /// <summary>
        /// Layout colPadding and rowPadding.
        /// </summary>
        [Test]
        public void Layout_ColPaddingRowPadding()
        {
            manager = new FixedGridLayoutManager(0, 50);
            Dependency dependency = new Dependency("Dependency Map Renderer");
            dependency.AddDependant(new Dependency("SVG Generator"));
            dependency.AddDependant(new Dependency("Base Exception"));

            graphics = new SVGGraphicsAdapter(filename + "colpadding_rowpadding.svg");

            manager.Layout(dependency, factory, graphics);
            graphics.Complete();
        }

        /// <summary>
        /// Layout textMargin.
        /// </summary>
        [Test]
        public void Layout_TextMargin()
        {
            manager = new FixedGridLayoutManager(10, 10, 50);
            Dependency dependency = new Dependency("Dependency Map Renderer");
            dependency.AddDependant(new Dependency("SVG Generator"));
            dependency.AddDependant(new Dependency("Base Exception"));
            
            graphics = new SVGGraphicsAdapter(filename + "text_margin.svg");
            
            manager.Layout(dependency, factory, graphics);
            graphics.Complete();
        }

        /// <summary>
        /// Test Layout() with null dependency.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Layout_NullDependency()
        {
            graphics = new SVGGraphicsAdapter(filename + "fail_layout.svg");
            manager.Layout(null, factory, graphics);
        }

        /// <summary>
        /// Test Layout() with null shape factory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Layout_NullShapeFactory()
        {
            graphics = new SVGGraphicsAdapter(filename + "fail_factory.svg");
            manager.Layout(new Dependency("a"), null, graphics);
        }

        /// <summary>
        /// Test Layout() with null renderer graphics.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Layout_NullRendererGraphics()
        {
            manager.Layout(new Dependency("a"), factory, null);
        }

    }

}
