/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import junit.framework.TestCase;
import java.io.File;

/**
 * Unit test cases for DependencyRenderer.
 *
 * Testing of DependencyRenderer only handles the properties. The accuracy of the rendering algorithm
 * is tested in the FixedGridLayoutManager.
 *
 * @author TCSDeveloper
 */
public class DependencyRendererUnitTests extends TestCase {

    /**
     * The filename to persist to.
     */
    private String filename = "test_files/unittest.svg";

    /**
     * The shape factory to use.
     */
    private ShapeFactory factory = null;

    /**
     * The diagram renderer to use.
     */
    private DiagramRenderer dr = null;

    /**
     * A second diagram renderer to use.
     */
    private DiagramRenderer dr2 = null;

    /**
     * The layout manager to use.
     */
    private LayoutManager manager = new FixedGridLayoutManager();

    /**
     * The dependency renderer instance to test.
     */
    private DependencyRenderer renderer = null;

    /**
     * Create instance of shape factory, diagram renderer and dependency renderer for each test.
     */
    protected void setUp() {
        factory = new ShapeFactory();
        dr = new SVGRenderer(new File(filename));
        dr2 = new SVGRenderer(new File(filename));
        renderer = new DependencyRenderer(factory);
    }

    /**
     * Create instance.
     */
    public void testCreate() {
        renderer = new DependencyRenderer();
        assertNotNull("Layout manager not set.", renderer.getLayoutManager());
        assertEquals("Diagram renderers should be empty.", 0, renderer.getDiagramRenderers().size());
    }

    /**
     * Create instance with shape factory.
     */
    public void testCreate_ShapeFactory() {
        renderer = new DependencyRenderer(factory);
        assertNotNull("Layout manager not set.", renderer.getLayoutManager());
        assertEquals("Diagram renderers should be empty.", 0, renderer.getDiagramRenderers().size());
    }

    /**
     * Create instance with null shape factory.
     */
    public void testCreate_NullShapeFactory() {
        try {
            new DependencyRenderer(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test addDiagramRenderer().
     */
    public void testAddDiagramRenderer() {
        renderer.addDiagramRenderer(dr);

        assertEquals("Incorrect renderer count.", 1, renderer.getDiagramRenderers().size());
        assertTrue("Does not contain the renderer.", renderer.getDiagramRenderers().contains(dr));

        renderer.addDiagramRenderer(dr2);

        assertEquals("Incorrect renderer count.", 2, renderer.getDiagramRenderers().size());
        assertTrue("Does not contain the renderer.", renderer.getDiagramRenderers().contains(dr2));
    }

    /**
     * Test addDiagramRenderer() with null diagram renderer.
     */
    public void testAddDiagramRenderer_NullDiagramRenderer() {
        try {
            renderer.addDiagramRenderer(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test removeDiagramRenderer().
     */
    public void testRemoveDiagramRenderer() {
        renderer.addDiagramRenderer(dr);
        renderer.addDiagramRenderer(dr2);

        assertTrue("Diagram renderer not removed.", renderer.removeDiagramRenderer(dr));
        assertFalse("Diagram renderer removed again.", renderer.removeDiagramRenderer(dr));

        assertTrue("Diagram renderer not removed.", renderer.removeDiagramRenderer(dr2));
        assertFalse("Diagram renderer removed again.", renderer.removeDiagramRenderer(dr2));
    }

    /**
     * Test removeDiagramRenderer() with null diagram renderer.
     */
    public void testRemoveDiagramRenderer_NullDiagramRenderer() {
        try {
            renderer.removeDiagramRenderer(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test getDiagramRenderers().
     */
    public void testGetDiagramRenderers() {
        renderer.addDiagramRenderer(dr);
        renderer.addDiagramRenderer(dr2);

        assertEquals("Incorrect renderer count.", 2, renderer.getDiagramRenderers().size());
        assertTrue("Does not contain the renderer.", renderer.getDiagramRenderers().contains(dr));
        assertTrue("Does not contain the renderer.", renderer.getDiagramRenderers().contains(dr2));
    }

    /**
     * Test setLayoutManager().
     */
    public void testSetLayoutManager() {
        renderer.setLayoutManager(manager);
        assertEquals("Layout manager mismatches.", manager, renderer.getLayoutManager());
    }

    /**
     * Test setLayoutManager() with null layout manager.
     */
    public void testSetLayoutManager_NullLayoutManager() {
        try {
            renderer.setLayoutManager(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test getLayoutManager().
     */
    public void testGetLayoutManager() {
        renderer.setLayoutManager(manager);
        assertEquals("Layout manager mismatches.", manager, renderer.getLayoutManager());
    }

    /**
     * Test render().
     *
     * @throws RendererException to JUnit.
     */
    public void testRender() throws RendererException {
        renderer.addDiagramRenderer(dr);
        renderer.addDiagramRenderer(dr2);
        renderer.render(new Dependency("Dependency Map Renderer"));

        File output = new File(filename);
        assertTrue("Output does not exist.", output.exists());
        output.delete();
    }

    /**
     * Test render() with null dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testRender_NullDependency() throws RendererException {
        try {
            renderer.render(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }
    
    /**
     * Test renderAll(Dependency dependency).
     *
     * @throws RendererException to JUnit.
     */
    public void testRenderAllDependency() throws RendererException {
        renderer.addDiagramRenderer(dr);
        renderer.addDiagramRenderer(dr2);
        Dependency d = new Dependency("Dependency Map Renderer");
        d.addDependant(d);
        d.addDependant(new Dependency("Dependency Map Renderer"));
        renderer.renderAll(d);

        File output = new File(filename);
        assertTrue("Output does not exist.", output.exists());
        output.delete();
    }
    
    /**
     * Test render() with null dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testRenderAllDependency_NullDependency() throws RendererException {
        try {
            renderer.renderAll(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }
    
    /**
     * Test renderAll(Dependency dependency).
     *
     * @throws RendererException to JUnit.
     */
    public void testRenderAllDependencyBoolean() throws RendererException {
        renderer.addDiagramRenderer(dr);
        renderer.addDiagramRenderer(dr2);
        Dependency d = new Dependency("Dependency Map Renderer");
        d.addDependant(d);
        d.addDependant(new Dependency("Dependency Map Renderer"));
        renderer.renderAll(d, false);

        File output = new File(filename);
        assertTrue("Output does not exist.", output.exists());
        output.delete();
    }
    
    /**
     * Test render() with null dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testRenderAllDependencyBoolean_NullDependency() throws RendererException {
        try {
            renderer.renderAll(null, false);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }
}